#!/usr/bin/env python
# encoding: utf-8
import requests

from .base import BaseProvider


class MyMemoryProvider(BaseProvider):
    '''
    @MyMemoryProvider: This is a integration with Translated MyMemory API.
    Follow Informations:
      Website: https://mymemory.translated.net/
      Documentation: https://mymemory.translated.net/doc/spec.php

    Usage Tips: Use a valid email instead of the default.
        With a valid email you get 10 times more words/day to translate.
    For further infomations checkout:
    http://mymemory.translated.net/doc/usagelimits.php
                                                    Tips from: @Bachstelze
    '''
    base_url = 'http://api.mymemory.translated.net/get'

    def __init__(self, **kwargs):
        try:
            super().__init__(**kwargs)
        except TypeError:
            super(MyMemoryProvider, self).__init__(**kwargs)

        self.email = kwargs.get('email', '')
        self.languages = '{}|{}'.format(self.from_lang, self.to_lang)

    def _make_request(self, text):
        params = {'q': text, 'langpair': self.languages}
        if self.email:
            params['de'] = self.email

        response = requests.get(self.base_url, params=params, headers=self.headers)
        return response.json()

    def get_translation(self, text):
        data = self._make_request(text)

        translation = data['responseData']['translatedText']
        if translation:
            return translation
        else:
            matches = data['matches']
            next_best_match = next(match for match in matches)
            return next_best_match['translation']



#!/usr/bin/env python
# encoding: utf-8
from datetime import timedelta
from datetime import datetime

from lxml import etree
import requests

from .base import BaseProvider
from ..constants import TRANSLATION_FROM_DEFAULT


class AzureAuthClient:
    """
    Provides a client for obtaining an OAuth token from the authentication service
    for Microsoft Translator in Azure Cognitive Services.

    PS:
    token field is used to store the last token obtained from the token service
    the cached token is re-used until the time specified in reuse_token_until.
    """
    base_url = 'https://api.cognitive.microsoft.com/sts/v1.0/issueToken'

    def __init__(self, client_secret):
        self.client_secret = client_secret
        self.token = None
        self.reuse_token_until = None

    def get_access_token(self):
        '''
        Returns an access token for the specified subscription.

        This method uses a cache to limit the number of requests to the token service.
        A fresh token can be re-used during its lifetime of 10 minutes. After a successful
        request to the token service, this method caches the access token. Subsequent
        invocations of the method return the cached token for the next 5 minutes. After
        5 minutes, a new token is fetched from the token service and the cache is updated.
        '''

        if (self.token is None) or (datetime.utcnow() > self.reuse_token_until):
            headers = {'Ocp-Apim-Subscription-Key': self.client_secret}
            response = requests.post(self.base_url, headers=headers)
            response.raise_for_status()

            self.token = response.content
            self.reuse_token_until = datetime.utcnow() + timedelta(minutes=5)

        return self.token.decode('utf-8')


class MicrosoftProvider(BaseProvider):
    '''
    @MicrosoftProvider: This is a integration with Microsoft Translator API.
    Website: http://docs.microsofttranslator.com
    Documentation: http://docs.microsofttranslator.com/text-translate.html
    '''
    base_url = 'http://api.microsofttranslator.com/v2/Http.svc/Translate'

    def _make_request(self, text):
        auth_client = AzureAuthClient(self.secret_access_key)
        access_token = 'Bearer {}'.format(auth_client.get_access_token())
        self.headers.update({"Authorization ": access_token})

        params = {'text': text, 'to': self.to_lang}
        if self.from_lang != TRANSLATION_FROM_DEFAULT:
            params['from'] = self.from_lang

        response = requests.get(self.base_url, params=params, headers=self.headers)

        return response.text

    def get_translation(self, text):
        data = self._make_request(text)

        translation = etree.fromstring(data.encode('utf-8'))
        return translation.text



#!/usr/bin/env python
# encoding: utf-8

from abc import ABCMeta, abstractmethod


class BaseProvider:
    __metaclass__ = ABCMeta

    base_url = ''

    def __init__(self, to_lang, from_lang='en', secret_access_key=None, **kwargs):
        self.headers = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebit/535.19'
                                      '(KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19'}
        self.from_lang = from_lang
        self.to_lang = to_lang
        self.secret_access_key = secret_access_key

    @abstractmethod
    def get_translation(self, params):
        return NotImplemented('Please Implement this method')



#!/usr/bin/env python
# encoding: utf-8

from .mymemory_translated import MyMemoryProvider  # noqa
from .microsoft import MicrosoftProvider  # noqa

__all__ = ['MyMemoryProvider', 'MicrosoftProvider']



#!/usr/bin/env python
# encoding: utf-8
from textwrap import wrap

from .exceptions import InvalidProviderError
from .providers import MyMemoryProvider, MicrosoftProvider

DEFAULT_PROVIDER = MyMemoryProvider
TRANSLATION_API_MAX_LENGHT = 1000

PROVIDERS_CLASS = {
    'mymemory': MyMemoryProvider,
    'microsoft': MicrosoftProvider
}


class Translator:
    def __init__(self, to_lang, from_lang='en', provider=None, secret_access_key=None, **kwargs):
        self.available_providers = list(PROVIDERS_CLASS.keys())
        self.from_lang = from_lang
        self.to_lang = to_lang
        if provider and provider not in self.available_providers:
            raise InvalidProviderError(
                'Provider class invalid. '
                'Please check providers list bellow: {!r}'.format(self.available_providers)
            )

        provider_class = PROVIDERS_CLASS.get(provider, DEFAULT_PROVIDER)

        self.provider = provider_class(
            from_lang=from_lang,
            to_lang=to_lang,
            secret_access_key=secret_access_key,
            **kwargs
        )

    def translate(self, text):
        if self.from_lang == self.to_lang:
            return text

        text_list = wrap(text, TRANSLATION_API_MAX_LENGHT, replace_whitespace=False)
        return ' '.join(self.provider.get_translation(text_wraped) for text_wraped in text_list)



#!/usr/bin/env python
# encoding: utf-8
import os
import click
try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser
import locale
import sys

from .translate import Translator
from .version import __version__
from .constants import CONFIG_FILE_PATH, DEFAULT_PROVIDER, TRANSLATION_FROM_DEFAULT


here = os.path.dirname(os.path.abspath(__file__))


def get_config_info(lang_type):
    config_file_path = os.path.expanduser(CONFIG_FILE_PATH)
    lang_types = ('from_lang', 'to_lang')
    if not os.path.exists(config_file_path) or lang_type not in lang_types:
        return ''

    config_parser = ConfigParser()
    config_parser.read(config_file_path)
    default_section = 'DEFAULT'
    return config_parser.get(default_section, lang_type)


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return

    click.echo('translate, version {}'.format(__version__))
    ctx.exit()


@click.command()
@click.option(
    '--version',
    callback=print_version,
    expose_value=False,
    is_flag=True,
    is_eager=True,
    help='Show the version and exit.'
)
@click.option(
    'from_lang', '--from', '-f',
    default=get_config_info('from_lang') or TRANSLATION_FROM_DEFAULT,
    help="Sets the language of the text being translated. The default value is 'autodetect'"
)
@click.option(
    'to_lang', '--to', '-t',
    default=get_config_info('to_lang'),
    prompt='Translate to',
    help='Sets the language you want to translate.'
)
@click.option(
    'provider', '--provider', '-p',
    default=DEFAULT_PROVIDER,
    help="The providers name you wish to use. The default value is '{}'".format(DEFAULT_PROVIDER)
)
@click.option(
    'secret_access_key', '--secret_access_key',
    help="Set the secret access key used to get provider oAuth token",
    required=False,
)
@click.argument('text', nargs=-1, type=click.STRING, required=True)
def main(from_lang, to_lang, provider, secret_access_key, text):
    """
    Python command line tool to make on line translations

    \b
    Example:

    \t $ translate-cli -t zh the book is on the table
    \t 碗是在桌子上。

    \b
    Available languages:

    \t https://en.wikipedia.org/wiki/ISO_639-1
    \t Examples: (e.g. en, ja, ko, pt, zh, zh-TW, ...)
    """
    text = ' '.join(text)

    kwargs = dict(from_lang=from_lang, to_lang=to_lang, provider=provider)
    if provider != DEFAULT_PROVIDER:
        kwargs['secret_access_key'] = secret_access_key

    translator = Translator(**kwargs)
    translation = translator.translate(text)
    if sys.version_info.major == 2:
        translation = translation.encode(locale.getpreferredencoding())

    click.echo(translation)

    return translation
